package org.saev.deob;

import java.util.*;
import java.io.File;
import java.io.IOException;
import org.jdom.*;
import org.jdom.input.*;

public class Main {

	public static void main(String[] args) {
		try {
			String opt = args[0].toLowerCase();

			if(opt.equals("list")) {
				list();
			} else if(opt.equals("info")) {
				info(args[1]);
			} else if(opt.equals("apply")) {
				apply(new File(args[1]));
			} else if(opt.equals("help")) {
				printUsage();
			} else {
				System.err.println(opt + " is not a reconized command");
				printUsage();
			}
		} catch(ArrayIndexOutOfBoundsException e) {
			printUsage();
		}
	}

	private static void list() {
		System.out.println("listing transforms...");
		Transformer[] trans = TransformerFactory.getTransformers();

		for(int i = 0; i < trans.length; i++) {
			System.out.println("[" + (i + 1) + "]\t" + trans[i].getName());
		}
	}

	private static void info(String transName) {
		Transformer t = TransformerFactory.getTransformer(transName);

		if(t != null) {
			System.out.println("info for transformer " + transName + ":");
			System.out.println("description: " + t.getDescription());
			System.out.println("author: " + t.getAuthor());
		} else {
			System.out.println("transformer " + t + "could not be found");
		}
	}

	private static void apply(File f) {

		try {
			Document d = new SAXBuilder().build(f);
			Element root = d.getRootElement();
			String in = root.getChildText("inputJar");
			//System.out.println("in = " + in);
            Application app = new Application(in);
			List<Element> elements = root.getChild("transformers").getChildren();

			for(Element e : elements) {
				String transName = e.getAttributeValue("name");
				System.out.println("applying transform: " + transName);
				Transformer t = TransformerFactory.getTransformer(transName);
				t.transform(app);
			}
			String out = root.getChildText("outputJar");
			//System.out.println("out = " + out);
			app.dumpJar(out);
		} catch(Exception e) {
			System.err.println("fatal: could not parse " + f);
			//e.printStackTrace();
			System.exit(1);
		}
	}

	private static void printUsage() {
		System.out.println("usage: deob [option]");
		System.out.println("options: ");
		System.out.println("\tlist\t\tdisplays all the available transforms.");
		System.out.println("\tinfo [name]\tdisplays some information about a transform.");
		System.out.println("\tapply [xml]\truns the transforms specified in an xml file.");
		System.out.println("\thelp\t\tdisplay this message.");
		System.out.println("\nsee the README file for more information.");
		System.out.println("deob by saevion (with much help from Sean as always).");
	}
}